@@ -9,16 +9,9 @@ from pydantic import BaseModel
 
 from ._operations import exec_query, expand_values
 from ._transactions import Transaction
-from .types import Params, PydanticParams, Query
+from .types import AsyncConnectionPatch, Params, PydanticParams, Query
 
-T = TypeVar("T", bound=BaseModel)
-
-
-async def _configure(con: psycopg.AsyncConnection) -> None:
-    """
-    Configure the Connections in the pool.
-    """
-    await con.set_autocommit(True)
+T = TypeVar("T", bound=(BaseModel | dict))
 
 
 class Postgres:
@@ -31,6 +24,7 @@ class Postgres:
         database: str = "postgres",
         pool_min_size: int = 10,
         pool_max_size: int = 50,
+        patch: Optional[AsyncConnectionPatch] = None,
         name: str = "python-postgres",
         timeout: float = 30.0,
         max_waiting: int = 0,
@@ -52,6 +46,8 @@ class Postgres:
         :param database: The database name to connect to, default is `postgres`.
         :param pool_min_size: The minimum number of connections to keep in the pool.
         :param pool_max_size: The maximum number of connections to keep in the pool.
+        :param patch: A list of functions to call on the connection after it is created. This is
+               useful to set up the connection with custom settings, like enabling extensions.
         :param name: An optional name to give to the connection pool, to identify it in the logs.
                      Default to `python-postgres` to distinguish it from other pools that may be
                      active.
@@ -64,13 +60,20 @@ class Postgres:
         :param reconnect_timeout: The timeout in seconds to wait for a connection to be reconnected
                                     from the pool. Default is 5 minutes.
         """
+
+        async def configure(con: psycopg.AsyncConnection) -> None:
+            if patch:
+                for func in patch:  # We cannot assume concurrency is safe.
+                    await func(con)
+            await con.set_autocommit(True)
+
         self._uri = f"postgresql://{user}:{quote_plus(password)}@{host}:{port}/{database}"
         self._pool = AsyncConnectionPool(
             self._uri,
             min_size=pool_min_size,
             max_size=pool_max_size,
             open=False,
-            configure=_configure,
+            configure=configure,
             name=name,
             timeout=timeout,
             max_waiting=max_waiting,
@@ -83,18 +86,19 @@ class Postgres:
     @overload
     async def __call__(
         self, query: Query, params: Params = (), *, model: Type[T], **kwargs
-    ) -> list[T] | int: ...
+    ) -> list[T]: ...
 
     @overload
-    async def __call__(self, query: Query, params: Params = (), **kwargs) -> list[tuple] | int: ...
+    async def __call__(self, query: Query, params: Params = (), **kwargs) -> list[tuple]: ...
 
     async def __call__(
         self,
         query: Query,
         params: Params = (),
+        binary: bool = True,
         model: Optional[Type[T]] = None,
         **kwargs,
-    ) -> list[T] | list[tuple] | int:
+    ) -> list[T] | list[tuple]:
         """
         Execute a query and return the results, or the number of affected rows. You can pass any
         query to this method. The Connections in the pool are in `autocommit` mode by default. This
@@ -105,8 +109,10 @@ class Postgres:
 
         :param query: The query to execute.
         :param params: The parameters to pass to the query.
-        :param model: The Pydantic model to parse the results into. If not provided, a new
-                      model with all columns in the query will be used.
+        :param binary: Whether to use binary mode for the cursor. Default is True, which is more
+               performant for most queries, but may not work with all queries. If you need to use
+               text mode i.e. for type adapters, set this to False.
+        :param model: The Pydantic model to parse the results into.
         :param kwargs: Keyword arguments passed to the Pydantic serialization method, such as
                `by_alias`, `exclude`, etc. This is usually the easiest way to make sure your model
                fits the table schema definition. **`exclude_none` is always set.**
@@ -115,22 +121,62 @@ class Postgres:
         await self._ensure_open()
         row_factory = class_row(model) if model else None
         async with self._pool.connection() as con:  # type: psycopg.AsyncConnection
-            async with con.cursor(binary=True, row_factory=row_factory) as cur:  # type: psycopg.AsyncCursor
+            async with con.cursor(binary=binary, row_factory=row_factory) as cur:  # type: psycopg.AsyncCursor
                 await exec_query(self._pool, cur, query, params, **kwargs)
-                return (
-                    cur.rowcount
-                    if not cur.pgresult or not cur.description or cur.rowcount == 0
-                    else await cur.fetchall()
-                )
+                return await cur.fetchall()
+
+    @overload
+    async def one(
+        self, query: Query, params: Params = (), *, model: Type[T], **kwargs
+    ) -> T | None: ...
+
+    @overload
+    async def one(self, query: Query, params: Params = (), **kwargs) -> tuple | None: ...
+
+    async def one(
+        self,
+        query: Query,
+        params: Params = (),
+        binary: bool = True,
+        model: Optional[Type[T]] = None,
+        **kwargs,
+    ) -> T | tuple | None:
+        """
+        Execute a query and return the first result, or None if no results are found. Otherwise,
+        this behaves identically to the `__call__` method.
+        :param query: The query to execute.
+        :param params: The parameters to pass to the query.
+        :param model: The Pydantic model to parse the results into. If not provided, a new
+                      model with all columns in the query will be used.
+        :param binary: Whether to use binary mode for the cursor. Default is True, which is more
+               performant for most queries, but may not work with all queries. If you need to use
+               text mode i.e. for type adapters, set this to False.
+        :param kwargs: Keyword arguments passed to the Pydantic serialization method, such as
+               `by_alias`, `exclude`, etc. This is usually the easiest way to make sure your model
+               fits the table schema definition. **`exclude_none` is always set.**
+        :return: The first result of the query, or None if there isn't one.
+        """
+        await self._ensure_open()
+        row_factory = class_row(model) if model else None
+        async with self._pool.connection() as con:  # type: psycopg.AsyncConnection
+            async with con.cursor(binary=binary, row_factory=row_factory) as cur:  # type: psycopg.AsyncCursor
+                await exec_query(self._pool, cur, query, params, **kwargs)
+                return await cur.fetchone()
 
     async def insert(
-        self, table_name: LiteralString, params: PydanticParams, prepare: bool = False, **kwargs
-    ) -> int:
+        self,
+        table_name: LiteralString,
+        params: PydanticParams,
+        prepare: bool = False,
+        binary: bool = True,
+        returning: Optional[list[LiteralString]] = None,
+        **kwargs,
+    ) -> list[tuple] | tuple | int:
         """
         Dynamically expand an insert query to correctly handle pydantic models with optional
         fields, applying default values rather than explicitly passing `None` to the query. This
         always produces one single Query. The column names to insert are determined by all the
-        non-None across all given models.
+        non-`None` fields across all given models.
 
         This will not be particularly efficient for large inserts and solves a specific problem. If
         you have uniform models and can construct one query to achieve the same, you should prefer
@@ -140,30 +186,42 @@ class Postgres:
         :param params: The Pydantic model or list of models to insert.
         :param prepare: Whether to use prepared statements. Default is False, due to the dynamic
                         nature and possibly rather large size of the query.
+        :param binary: Whether to use binary mode for the cursor. Default is True, which is more
+               performant for most queries, but may not work with all queries. If you need to use
+               text mode i.e. for type adapters, set this to False.
+        :param returning: An optional list of Column Names to return after the insert.
         :param kwargs: Keyword arguments passed to the Pydantic serialization method, such as
                `by_alias`, `exclude`, etc. This is usually the easiest way to make sure your model
                fits the table schema definition. **`exclude_none` is always set.**
-        :return: The number of rows inserted.
+        :return: If `returning` is provided, returns the specified columns. Otherwise, returns the
+                 number of rows affected by the insert.
         """
-        if isinstance(params, list) and not params:
+        is_multiple = isinstance(params, list)
+        if is_multiple and not params:
             return 0
         await self._ensure_open()
-        query, params = expand_values(table_name, params, **kwargs)
+        query, params = expand_values(table_name, params, returning, **kwargs)
         async with self._pool.connection() as con:  # type: psycopg.AsyncConnection
-            async with con.cursor(binary=True) as cur:  # type: psycopg.AsyncCursor
+            async with con.cursor(binary=binary) as cur:  # type: psycopg.AsyncCursor
                 await cur.execute(query, params, prepare=prepare)
+                if returning:
+                    return await cur.fetchall() if is_multiple else await cur.fetchone()
                 return cur.rowcount
 
     @asynccontextmanager
-    async def transaction(self) -> AsyncIterator[Transaction]:
+    async def transaction(self, binary: bool = True) -> AsyncIterator[Transaction]:
         """
         Create a transaction context manager to execute multiple queries in a single transaction.
         You can call the transaction the same way you would call the `Postgres` instance itself.
+
+        :param binary: Whether to use binary mode for the cursor. Default is True, which is more
+               performant for most queries, but may not work with all queries. If you need to use
+               text mode i.e. for type adapters, set this to False.
         """
         await self._ensure_open()
         async with self._pool.connection() as con:  # type: psycopg.AsyncConnection
             async with con.transaction():
-                async with con.cursor(binary=True) as cur:  # type: psycopg.AsyncCursor
+                async with con.cursor(binary=binary) as cur:  # type: psycopg.AsyncCursor
                     yield Transaction(self._pool, cur)
 
     @asynccontextmanager
